-- src/RandomUtils.hs
module RandomUtils (generateUniformPoints, generateNormalPoints) where

import System.Random
import Control.Monad (replicateM)

-- Generate random points from a uniform distribution in the range [a, b]
generateUniformPoints :: Int -> (Double, Double) -> IO [Double]
generateUniformPoints n (a, b) = replicateM n (randomRIO (a, b))

-- Generate random points from a normal distribution using the Box-Muller transform
generateNormalPoints :: Int -> Double -> Double -> IO [Double]
generateNormalPoints n mu sigma = do
    gen <- newStdGen
    let randomFloats = take (2 * n) $ randomRs (0, 1) gen
    return $ boxMullerTransform mu sigma randomFloats

-- Box-Muller transform to generate normally distributed random values
boxMullerTransform :: Double -> Double -> [Double] -> [Double]
boxMullerTransform mu sigma (u1:u2:us) = 
    let z0 = sqrt (-2 * log u1) * cos (2 * pi * u2)
        z1 = sqrt (-2 * log u1) * sin (2 * pi * u2)
    in (mu + z0 * sigma) : (mu + z1 * sigma) : boxMullerTransform mu sigma us
boxMullerTransform _ _ _ = []  -- If input list is not in pairs, return empty list
